#!/usr/bin/python
#
# (c) Max Kresch, licensed under GPLv3, 2008.
#     See LICENSE for further details. 

#---docstring------------------------------------------------------------------
""" 
Some physical constants in mks units, and some conversions to units 
convenient for neutron scattering.

A2m   = multiply by this to take Angstroms to meters
J2meV = multiply by this to convert Joules to meV 
amu   = mass of `atomic mass unit`
h     = Planck's constant
h_b   = Planck's constant / 2 pi
k_b   = Boltzmann's constant 
m_n   = Neutron mass
"""
#---end-docstring--------------------------------------------------------------

A2m   =   1.0e-10         # multiplication by takes Angstroms to meters
J2meV =   6.2415097e+21   # multiplication by takes Joules to meV
amu   =   1.66053873e-27  # kg
h     =   6.6260688e-34   # kg m^2 / s
h_b   =   1.0545716e-34   # kg m^2 / s
k_b   =   1.3806503e-23   # kg m^2 / K s^2
m_n   =   1.6749272e-27   # kg

